﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Reflection;
using System.CodeDom.Compiler;
using Microsoft.CSharp;

namespace ScorchedEarth
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            expressionsListBox.SelectionMode = SelectionMode.Single;
            expressionsListBox.IsSynchronizedWithCurrentItem = true;
            addButton.Click += new RoutedEventHandler(addNewExpressionClick);
            deleteButton.Click += new RoutedEventHandler(deleteButton_Click);

            MyDataContext myDataContext = new MyDataContext();
            this.DataContext = myDataContext;

            foreach (var expression in myDataContext.Expressions)
            {
                expression.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(expression_PropertyChanged);
            }
            expressionsListBox.SelectedIndex = 1;
        }

        void expression_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            Expression expression = sender as Expression;
            if (expression != null)
            {
                this.statusBox.Text = expression.CompilerMessage;
                CreateImage(expression);
            }
        }


        private void CreateImage(Expression expression)
        {
            if (expression == null)
                return;
                // Get the Func<double,double> associated with this expression.
            if (expression.Function == null)
                return;
  
            var heightFunction = expression.Function;

            // Define parameters used to create the BitmapSource.
            PixelFormat pf = PixelFormats.Bgr32;
            int width = 800;
            int height = 600;
            int rawStride = (width * pf.BitsPerPixel + 7) / 8;
            //byte[] rawImage = new byte[rawStride * height];
            int[] rawImage = new int[width*height];

            int skyColor = 0x000077;
            int groundColor = 0x11aa11;
            // Initialize the image with data.
            for (int x = 0; x < width; x++)
            {
                double terrain = heightFunction((double)x / (double)(width-1));
                int groundY = Math.Max(0,Math.Min(height-1,(int)((1.0f - terrain) * (double)(height-1))));
                for (int y = 0; y < groundY; y++)
                {
                    rawImage[y * width + x] = skyColor;
                }
                for (int y = groundY; y < height; y++)
                {
                    rawImage[y * width + x] = groundColor;
                }

            }
                //for (int index = 0; index < width * height; index++)
                //    rawImage[index] = 0x070377;
            //Random value = new Random();
            //value.NextBytes(rawImage);

            // Create a BitmapSource.
            BitmapSource bitmap = BitmapSource.Create(width, height,
                96, 96, pf, null,
                rawImage, rawStride);

            this.resultImage.Source = bitmap;
        }

        void deleteButton_Click(object sender, RoutedEventArgs e)
        {
            int index = expressionsListBox.SelectedIndex;
            MyDataContext context = this.DataContext as MyDataContext;
            Expression expression = context.Expressions[index];
            expression.PropertyChanged -= new System.ComponentModel.PropertyChangedEventHandler(expression_PropertyChanged);
            context.Expressions.RemoveAt(index);
        }

        static int count = 0;
        void addNewExpressionClick(object sender, RoutedEventArgs e)
        {
            MyDataContext context = this.DataContext as MyDataContext;
            Expression expression = new Expression();
            expression.Name = "NewExpression" + (++count).ToString();
            Parameter param = new Parameter() { Name = "A", Value = 0.0f };
            expression.Parameters = new ObservableCollection<Parameter>() { param };
            expression.Statement = "// TODO: Set the height";
            context.Expressions.Add(expression);
            expression.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(expression_PropertyChanged);
        }

        private void expressionsListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                Expression expression = e.AddedItems[0] as Expression;
                this.statusBox.Text = expression.CompilerMessage;
                CreateImage(expression);
            }
        }

        private void compileButton_Click(object sender, RoutedEventArgs e)
        {
            Expression expression = this.expressionsListBox.SelectedItem as Expression;
            if (expression != null)
            {
                expression.Statement = this.codeTextBlock.Text;
                this.statusBox.Text = expression.CompilerMessage;
                CreateImage(expression);
            }

        }


        //private Dictionary<string, Func<double, double>> functionMap = new Dictionary<string, Func<double, double>>();
    }
}
